#include "stdafx.h"


#include "editor_schema.h"
/**
 * @file  editor_schema.cpp
 * @brief 
 */

using namespace grt;
using namespace bec;

SchemaEditorBE::SchemaEditorBE(GRTManager *grtm, const db_SchemaRef &schema, const db_mgmt_RdbmsRef &rdbms)
  : DBObjectEditorBE(grtm, schema, rdbms), _schema(schema)
{
}

void SchemaEditorBE::set_schema_option_by_name(const std::string& name, const std::string& value)
{
  if(name.compare("CHARACTER SET - COLLATE") == 0)
  { // shortcut that sets both CHARACTER SET and COLLATE separated by a - 
    if (value != get_schema_option_by_name(name))
    {
      std::string charset, collation;
      parse_charset_collation(value, charset, collation);
      if (charset != *_schema->defaultCharacterSetName() || collation != *_schema->defaultCollationName())
      {
        block_auto_refresh();
        begin_undo_group();
        set_schema_option_by_name("CHARACTER SET", charset);
        set_schema_option_by_name("COLLATE", collation);
        update_change_date();
        end_undo_group(fmt(_("Change Charset/Collation for '%s'"), _schema->name().c_str()));
        unblock_auto_refresh();
      }
    }
  }
  else if(name.compare("CHARACTER SET") == 0)
  {
    if (begin_undo_object_text_change(get_schema(), "defaultCharacterSetName"))
    {
      update_change_date();
      end_undo_group(fmt(_("Set Default Character Set for Schema '%s'"), get_name().c_str()));
    }
    get_schema()->defaultCharacterSetName(value);
  }
  else if(name.compare("COLLATE") == 0)
  {
    if (begin_undo_object_text_change(get_schema(), "defaultCharacterSetName"))
    {
      update_change_date();
      end_undo_group(fmt(_("Set Default Collation for Schema '%s'"), get_name().c_str()));
    }
    get_schema()->defaultCollationName(grt::StringRef(value));
  }
}

std::string SchemaEditorBE::get_schema_option_by_name(const std::string& name)
{
  if(name.compare("CHARACTER SET") == 0)
    return get_schema()->defaultCharacterSetName();
  else if(name.compare("COLLATE") == 0)
    return get_schema()->defaultCollationName();

  return std::string();
}
