#include "stdafx.h"

#include <grts/structs.db.h>

#include <grtpp_util.h>
#include <grtpp_undo_manager.h>
#include "grt/common.h"

//================================================================================
// db_Schema


class db_Schema::ImplData
{
};


void db_Schema::init()
{
}

void db_Schema::set_data(ImplData *data)
{
  _data= data;
}

db_Schema::~db_Schema()
{
}


db_RoutineRef db_Schema::addNewRoutine(const std::string &dbpackage)
{
  grt::UndoManager *um= 0;
  db_RoutineRef routine;
  std::string class_name;

  if (is_global() && get_grt()->tracking_changes())
    um= get_grt()->get_undo_manager();

  class_name= dbpackage+".Routine";

  std::string name= grt::get_name_suggestion_for_list_object(grt::ObjectListRef::cast_from(_routines), "routine");

  routine= get_grt()->create_object<db_Routine>(class_name);
  routine->owner(this);
  routine->name(name);
  routine->createDate(bec::fmttime(0, DATETIME_FMT));
  routine->lastChangeDate(bec::fmttime(0, DATETIME_FMT));

  _routines.insert(routine);

  if (um)
  {
    um->set_action_description(_("Add New Routine Object"));
  }

  return routine;
}


db_RoutineGroupRef db_Schema::addNewRoutineGroup(const std::string &dbpackage)
{
  grt::UndoManager *um= 0;
  db_RoutineGroupRef rgroup;
  std::string class_name;

  if (is_global() && get_grt()->tracking_changes())
    um= get_grt()->get_undo_manager();

  class_name= dbpackage+".RoutineGroup";

  std::string name= grt::get_name_suggestion_for_list_object(grt::ObjectListRef::cast_from(routineGroups()), "routines");

  rgroup= get_grt()->create_object<db_RoutineGroup>(class_name);
  rgroup->owner(this);
  rgroup->name(name);
  rgroup->createDate(bec::fmttime(0, DATETIME_FMT));
  rgroup->lastChangeDate(bec::fmttime(0, DATETIME_FMT));

  _routineGroups.insert(rgroup);
  if (um)
  {
    um->set_action_description(_("Add New Routine Group Object"));
  }

  return rgroup;
}


db_TableRef db_Schema::addNewTable(const std::string &dbpackage)
{
  grt::UndoManager *um= 0;
  db_TableRef table;
  std::string class_name;

  if (is_global() && get_grt()->tracking_changes())
    um= get_grt()->get_undo_manager();

  class_name= dbpackage+".Table";

  std::string name= grt::get_name_suggestion_for_list_object(grt::ObjectListRef::cast_from(_tables), "table");

  table= get_grt()->create_object<db_Table>(class_name);
  table->owner(this);
  table->name(name);
  table->createDate(bec::fmttime(0, DATETIME_FMT));
  table->lastChangeDate(bec::fmttime(0, DATETIME_FMT));

  _tables.insert(table);
  if (um)
  {
    um->set_action_description(_("Add New Table Object"));
  }

  return table;
}


db_ViewRef db_Schema::addNewView(const std::string &dbpackage)
{
  grt::UndoManager *um= 0;
  db_ViewRef view;
  std::string class_name;

  if (is_global() && get_grt()->tracking_changes())
    um= get_grt()->get_undo_manager();

  class_name= dbpackage+".View";

  std::string name= grt::get_name_suggestion_for_list_object(grt::ObjectListRef::cast_from(_views), "view");

  view= get_grt()->create_object<db_View>(class_name);
  view->owner(this);
  view->name(name);
  view->createDate(bec::fmttime(0, DATETIME_FMT));
  view->lastChangeDate(bec::fmttime(0, DATETIME_FMT));

  _views.insert(view);
  if (um)
  {
    um->set_action_description(_("Add New View Object"));
  }

  return view;
}


grt::ListRef<db_ForeignKey> db_Schema::getForeignKeysReferencingTable(const db_TableRef &table)
{
  // from db_ForeignKey.cpp
  extern grt::ListRef<db_ForeignKey> get_foreign_keys_referencing_table(const db_TableRef &value);

  return get_foreign_keys_referencing_table(table);
}


