
#include "stdafx.h"

#include "wb_component.h"
#include "wb_model_view_form.h"
#include "wb_layer_tree.h"
#include <grtpp_undo_manager.h>

using namespace bec;
using namespace wb;

WBComponent::WBComponent(WBContext *context)
: _wb(context)
{
}



grt::ValueRef WBComponent::place_object_grt(grt::GRT *grt, ModelViewForm *form, const mdc::Point &pos, 
                                                const std::string &object_struct,
                                                const grt::DictRef &args)
{
  model_ViewRef view(form->get_model_view());
  std::string object_type, object_struct_name;

  mdc::Point offset;
  model_LayerRef layer= form->get_layer_at(pos, offset);

  if (!layer.is_valid())
    layer= view->rootLayer();

  form->get_view()->snap_to_grid(offset);

  model_FigureRef figure(grt->create_object<model_Figure>(object_struct));

  if (!figure.is_valid())
    throw std::runtime_error("Could not create object of type "+object_struct);

  figure->owner(view);
  figure->layer(layer);
  figure->left(offset.x);
  figure->top(offset.y);
  if (figure->name() == "")
    figure->name(figure.get_metaclass()->get_attribute("caption"));

  if (args.is_valid())
  {
    for (grt::DictRef::const_iterator iter= args.begin(); iter != args.end(); ++iter)
    {
      figure.set_member(iter->first, iter->second);
    }
  }

  if (!args.is_valid() || !args.has_key("color"))
  {
    if (!form->get_tool_argument(object_struct+":Color").empty())
      figure->color(grt::StringRef(form->get_tool_argument(object_struct+":Color")));
    else
      figure->color(_wb->get_wb_options().get_string(object_struct+":Color", ""));
  }

  _wb->get_grt()->begin_auto_undo();
  
  view->addFigure(figure);
  
  // ensure the figure fits in the layer and if not, resize the layer
  double w= *figure->width();
  double h= *figure->height();

  if (offset.x + w > *layer->width())
  {
    layer->width(offset.x + w);
  }

  if (offset.y + h > *layer->height())
  {
    layer->height(offset.y + h);
  }

  _wb->get_grt()->end_auto_undo(_("Place Figure"));

  // auto-select figure
  view->unselectAll();
  view->selectObject(figure);

  // manually notify selection change
  _wb->request_refresh(RefreshSelection, "", (NativeHandle)form->get_frontend_data());

  return figure;
}


grt::ValueRef WBComponent::place_object(ModelViewForm *view, const mdc::Point &pos, const std::string &object_struct,
                                            const grt::DictRef &args)
{
  return _wb->execute_in_grt_thread("Place object in model",
    sigc::bind<ModelViewForm*,mdc::Point,std::string,grt::DictRef>(sigc::mem_fun(this, &WBComponent::place_object_grt),
                                                                     view,
                                                                     pos,
                                                                     object_struct,
                                                                     args));
}



std::string WBComponent::get_command_option_value(const std::string &option)
{
  ModelViewForm *form= dynamic_cast<ModelViewForm*>(_wb->get_active_main_form());

  if (form)
    return form->get_tool_argument(option);

  return "";
}


void WBComponent::set_command_option_value(const std::string &option, const std::string &item)
{
  ModelViewForm *form= dynamic_cast<ModelViewForm*>(_wb->get_active_main_form());

  if (form)
    form->set_tool_argument(option, item);
}

