


#include "grtpp.h"
#include "grtpp_module_cpp.h"
#include "grtpp_util.h"

using namespace grt;


//--------------------------------------------------------------------------------
// Module Loader



//--------------------------------------------------------------------------------
// Modules


Module::Module(ModuleLoader *loader)
  : _loader(loader)
{
}


ValueRef Module::call_function(const std::string &name, const grt::BaseListRef &args)
{
  const Function *f= get_function(name);

  if (!f)
    throw grt::module_error(std::string("Module ").append(_name).append(" doesn't have function ").append(name));

  return f->call(args);
}




void Module::add_function(const Function &func)
{
  _functions.push_back(func);
}



const Module::Function *Module::get_function(const std::string &name) const
{
  for (std::vector<Function>::const_iterator iter= _functions.begin();
       iter != _functions.end(); ++iter)
  {
    if (iter->name == name)
      return &*iter;
  }

  if (!_extends.empty())
  {
    Module *module= get_grt()->get_module(_extends);
    if (!module)
      throw std::runtime_error(strfmt("Parent module '%s' of module '%s' was not found", _extends.c_str(), _name.c_str()));
    
    return module->get_function(name);
  }
  return 0;
}


void Module::validate() const
{
  if (name().empty()) throw std::runtime_error("Invalid module");

  // validate if registered functions conform to interfaces
  for (Interfaces::const_iterator iter= _interfaces.begin();
       iter != _interfaces.end(); ++iter)
  {
    const Interface *iface= get_grt()->get_interface(*iter);
    if (iface)
    {
      if (!iface->check_conformance(this))
        throw std::logic_error(std::string("Module ").append(name()).append(" does not conform to interface ").append(*iter));
    }
    else
      g_warning("Interface '%s' implemented by module '%s' is not registered",
                iter->c_str(), name().c_str());
  }
}

