
#include "grtui/gui_plugin_base.h"
#include "grtui/grt_wizard_form.h"
#include "grtui/wizard_finished_page.h"
#include "grtui/wizard_view_text_page.h"

#include "grtui/grtdb_object_filter.h"

#include "db_mysql_sql_script_sync.h"


using namespace grtui;
using namespace mforms;


#include "db/frontend/catalog_validation_page.h"
//#include "db/frontend/synchronize_direction_page.h"





class ExportInputPage : public WizardPage
{
public:
  ExportInputPage(WizardForm *form)
    : WizardPage(form, "options"), _options(true), _options_box(false),
     _in_box(true), _out_box(true)
  {
    set_title(_("Forward Engineer Options"));
    set_subtitle(_("Set various options for database to be created."));

    _options.set_title(_("Options"));

    _options.add(&_options_box);
    _options_box.set_padding(12);
    _options_box.set_spacing(8);

    _options_box.add(&_in_descr, false, false);
    _options_box.add(&_in_box, false, false);
    _in_box.add(&_in_label, false, false);
    _in_box.add(&_in_entry, true, true);
    _in_box.add(&_in_button, false, false);
    _in_descr.set_text(_("Pick the CREATE script file to be compared with the model."));
    _in_label.set_text(_("Input File:"));
    _in_button.set_text(_("Browse..."));
    enable_file_browsing(&_in_entry, &_in_button, mforms::OpenFile, "SQL Scripts (*.sql)");

    _options_box.add(&_out_descr, false, false);
    _options_box.add(&_out_box, false, false);
    _out_box.add(&_out_label, false, false);
    _out_box.add(&_out_entry, true, true);
    _out_box.add(&_out_button, false, false);
    _in_descr.set_text(_("Enter the path for the ALTER script to be created."));
    _out_label.set_text(_("Output File:"));
    _out_button.set_text(_("Browse..."));
    enable_file_browsing(&_out_entry, &_out_button, mforms::SaveFile, "SQL Scripts (*.sql)");

    add(&_options, false, false);
    
    signal_leave().connect(sigc::mem_fun(this, &ExportInputPage::gather_options));
  }

  void gather_options(grt::DictRef values)
  {    
    values.gset("InputPath", _in_entry.get_string_value());
    values.gset("OutputPath", _out_entry.get_string_value());
  }
  
  virtual bool allow_next()
  {
    return !_in_entry.get_string_value().empty() && !_out_entry.get_string_value().empty();
  }

protected:
  Panel _options;
  Box _options_box;
  
  Box _in_box;
  Label _in_label;
  Label _in_descr;
  TextEntry _in_entry;
  Button _in_button;

  Box _out_box;
  Label _out_label;
  Label _out_descr;
  TextEntry _out_entry;
  Button _out_button;
};







//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------

class WbPluginSQLSynchronize : public GUIPluginBase, public WizardForm
{  
public:
  WbPluginSQLSynchronize(bec::GRTManager *grtm, const grt::BaseListRef &args)
    : GUIPluginBase(grtm, args), WizardForm(grtm)
  {
    set_title(_("Script Synchronization - Generate an ALTER Script"));
  }

  virtual void execute()
  {
    grt::DictRef values(GUIPluginBase::grtm()->get_grt());
    
    WizardForm::show(values);
  }


  virtual void show()
  {
    execute();
  }
  
  virtual void hide()
  {
    //XXX clean this mess by renaming the show(DictRef) method from WizardForm
    mforms::Form::show(false);
  }
};






extern "C" 
{
  GUIPluginBase *createWBPluginSQLSynchronize(bec::GRTManager *grtm, const grt::BaseListRef &args)
  {
    return new WbPluginSQLSynchronize(grtm, args);
  }
};

